#include <Types.h>
#include <Memory.h>
#include <Quickdraw.h>
#include <Fonts.h>
#include <Events.h>
#include <Menus.h>
#include <Windows.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <OSUtils.h>
#include <ToolUtils.h>
#include <SegLoad.h>
#include <Sound.h>
#include <Palettes.h>
#include <stdio.h>

#include "Mandelbrot.h"

double rMin = -2.01;
double rMax = 2.01;
double iMin = -2.01;
double iMax = 2.01;

// Globals 
Rect	windRect;
	
// Prototypes
void Initialize(void);
void SetColorPalette(WindowPtr w);

#define EXACT	0
#define GRAY_RAMP	32
#define ScreenDepth(gdh)	((*((*gdh)->gdPMap))->pixelSize)
GDHandle	gMyGDevice;
void SetColorPalette(WindowPtr w) {
	CTabHandle	tab = GetCTable(128);//GRAY_RAMP + ScreenDepth(gMyGDevice));
	PaletteHandle	newPal = NewPalette((*tab)->ctSize, tab, pmTolerant,
									EXACT);

	NSetPalette(w, newPal, pmAllUpdates);
	ActivatePalette(w);
}


int main(void) {
	Initialize();
	Rect offRect;
	GWorldPtr oldWorld, offscreen;
	GDHandle oldDev;
	
	offRect = windRect;
	OffsetRect(&offRect, -offRect.left, -offRect.top);
	
	// offscreen code
	GetGWorld(&oldWorld, &oldDev);
	if(noErr != NewGWorld(&offscreen, 16, &offRect, NULL, NULL, mapPix)) {
		ParamText("\pCouldn't create GWorld", NULL, NULL, NULL);
		Alert(128, NULL);
		return;
	}
	
	SetColorPalette((GrafPtr) offscreen);
	
	SetPort((GrafPtr) offscreen);
	
	long startTick = TickCount();
	mandelbrot (windRect.right - windRect.left, windRect.bottom - windRect.top, rMin, rMax, iMin, iMax);
	long endTick = TickCount();
	
	// offscreen code
	SetGWorld(oldWorld, oldDev);
	CopyBits(&((GrafPtr) offscreen)->portBits, &((GrafPtr) oldWorld)->portBits,
				&offRect, &offRect,srcCopy,NULL);
	
	long totalTick = endTick - startTick;

	char str[10];
	sprintf(str, "%f", totalTick/60.0);
	c2pstr( str );
	
	TextFont(1);
	TextSize(9);

	MoveTo(20,20);
	PmForeColor(0);
	
	DrawString( (unsigned char *) str );
	
	do {
	} while (!Button());

	return 0;	
}

// 
//	Initialize everything for the program, make sure we can run
//

void Initialize(void) {
	WindowPtr	mainPtr;
	OSErr		error;
	SysEnvRec	theWorld;
		
	//
	//	Test the computer to be sure we can do color.  
	//	If not we would crash, which would be bad.  
	//	If we cant run, just beep and exit.
	//

	error = SysEnvirons(1, &theWorld);
	if (theWorld.hasColorQD == false) {
		SysBeep(50);
		ExitToShell();					// If no color QD, we must leave.
	}
	
	// Initialize all the needed managers.
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();

	//
	//	Make a new window for drawing in, and it must be a color window.  
	//	The window is full screen size, made smaller to make it more visible.
	//
	windRect.top = kUpperLeft;
	windRect.left = kUpperLeft;
	windRect.bottom = kMaxScreenX + kUpperLeft;
	windRect.right = kMaxScreenY + kUpperLeft;
	
	mainPtr = NewCWindow(nil, &windRect, "\pAlmond Bread", true, documentProc, 
						(WindowPtr) -1, false, 0);
		
	SetPort(mainPtr);						// set window to current graf port
	//SetColorPalette(mainPtr);
}


